######################################################################
#####
#####    PONGsim (Python Overlay Network Graphical Simulator)
#####    Main module of the PONGsim framework
#####    Version: 6.2
#####
######################################################################
#####    
#####    PONGsim (Python Overlay Network Graphical Simulator)
#####    
#####    Copyright (c) 2005-2007 Nicklas Beijar
#####    
#####    Permission is hereby granted, free of charge, to any person
#####    obtaining a copy of this software and associated documentation
#####    files (the "Software"), to deal in the Software without
#####    restriction, including without limitation the rights to use,
#####    copy, modify, merge, publish, distribute, sublicense, and/or
#####    sell copies of the Software, and to permit persons to whom the
#####    Software is furnished to do so, subject to the following
#####    conditions:
#####    
#####    The above copyright notice and this permission notice shall be
#####    included in all copies or substantial portions of the Software.
#####    
#####    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#####    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
#####    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#####    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#####    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
#####    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#####    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
#####    OTHER DEALINGS IN THE SOFTWARE.
#####
######################################################################
#####
#####    Nicklas Beijar, nbeijar@netlab.tkk.fi
#####    Networking Laboratory, Helsinki University of Technology
##### 
######################################################################
#####
#####    To do:
#####    - (nothing)
#####
######################################################################

from Tkinter import *
import random, string, thread, math, time, os, bisect, sys, traceback

WORLDSIZEX = 1000
WORLDSIZEY = 800

DEFAULTTRANSMISSIONDELAY = 0.020

######################################################################
#####    S C E N A R I O
######################################################################
#####
#####    Stores a single scenario.  
#####
######################################################################

class Scenario:
    
    def __init__(self, name):
        self.data = {}
        self.filename = None
        self.name = name
        
    def get(self, param, default=None):
        if self.data.has_key(param):
            return self.data[param]
        return default 

    def set(self, param, value):
        self.data[param] = value

    def set_typechk(self, param, value):
        if value[0]=='"' and value[-1]=='"':
            value = value[1:-1]
        elif value[0]=='{' and value[-1]=='}':
            value = value
        elif value == "True":
            value = True
        elif value == "False":
            value = False
        else:
            try:
                value = float(value)
            except ValueError:
                print "WARNING: Non-recognized parameter type in '"+str(value)+"' - assuming string"
                value = value
        self.data[param] = value

    def getname(self):
        return self.name

    def getall(self):
        return self.data
    
    def getfilename(self):
        if self.filename: return self.filename
        pattern = self.get("file", self.name)
        if pattern == None: print "WARNING: No filename specified - using default"
        while string.find(pattern, "[") >= 0:
            start = string.find(pattern, "[")
            end = string.find(pattern, "]", start)
            if start < 0 or end < 0 or end<=start:
                print "WARNING: Illegal name pattern:",pattern
                return self.name
            param = pattern[start+1:end]
            value = str(self.get(param, "_"+param+"_"))
            if value[-2:] == ".0": value = value[:-2]
            value = string.replace(value, " ", "")
            pattern = pattern[:start] + value + pattern[end+1:]
        self.filename = pattern # cache
        return pattern
        
    # returns a new identical copy of this scenario
    def copy(self):
        newscenario = Scenario(self.name)
        for param in self.data.keys():
            newscenario.set(param, self.data[param])
        return newscenario

    # returns the (parameter, valuerange) of the first multiline in the scenario or (None, None) if not found. 
    # a multiline is a line in format "[value1 value2 value3]" for defining several values. 
    def findmultiline(self):
        for param in self.data.keys():
            try:
                if self.data[param][0] == "{" and self.data[param][-1] == "}":
                    return param, self.data[param][1:-1]
            except TypeError:
                pass
        return None, None

######################################################################
#####    S C E N A R I O M A N A G E R
######################################################################
#####
#####    Loads and stores the scenarios. Manages the queue of scenarios to run. 
#####
######################################################################

class ScenarioManager:
    
    def __init__(self):
        self.scenarios = []
        self.activescenario = None
        self.activerun = 0
        self.resetqueue()
        
    def loadall(self, path):
        for filename in os.listdir(path):
            #fields = string.splitfields(filename, " ")
            #if fields[0] == "scenario": 
            if string.find(filename, ".scenario.txt") > -1: 
                self.load(path, filename)
            
    def load(self, path, filename):
        #scenario = string.joinfields(fields[1:], " ")
        #fields = string.splitfields(scenario, ".")
        #scenario = string.joinfields(fields[:-1], ".")
        scenario = string.replace(filename, ".scenario.txt", "")
        print "Reading scenario "+scenario
        fp = open(path + filename, "r")
        scen = Scenario(scenario)
        while 1:
            line = fp.readline()
            if not line: break
            line = string.replace(string.replace(line, "\r", ""), "\n", "")
            fields = string.splitfields(line, "=")
            if len(line) > 0 and line[0] != "#":
                if len(fields) == 2:
                    value = fields[1]
                    scen.set_typechk(fields[0], value)
                else:
                    print "Unknown line in "+filename+": "+line
        fp.close()
        for multiscen in self.makemultiscenarios(scen):
            self.scenarios.append((scenario, multiscen))
        if self.activescenario == None: self.activescenario = scen

    def count(self):
        return len(self.scenarios)

    def makemultiscenarios(self, scenario):
        scenarios = []
        scenarios.append(scenario)
        changed = True
        while changed == True:
            changed = False
            for scenario in scenarios:
                param, value = scenario.findmultiline()
                if param != None:
                    changed = True
                    values = string.splitfields(value, " ")
                    scenario.set_typechk(param, values[0])
                    for value in values[1:]:
                        newscenario = scenario.copy()
                        newscenario.set_typechk(param, value)
                        scenarios.append(newscenario)
        return scenarios

    def enqueue(self, scenarioname, runnumber):
        for (name, scen) in self.scenarios:
            if name == scenarioname:
                self.queue.append((runnumber, scen))

    def resetqueue(self):
        self.queue = []

    def pop(self):
        if len(self.queue) == 0: return None
        (self.activerun, self.activescenario) = self.queue.pop(0)
        return self.activescenario

    def getscenarios(self):
        names = []
        for (name, scen) in self.scenarios:
            if name not in names: names.append(name)
        return names

    #def getscenario(self):
    #    return self.activescenario
        
    def getrunnumber(self):
        return self.activerun

    # legacy
    def get(self, param, default=None):
        if self.activescenario == None: return None
        return self.activescenario.get(param, default)

    def getactivescenario(self):
        return self.activescenario

######################################################################
#####    C O U N T E R
######################################################################
#####
#####    Statistics utility. Independent (not related to simulator and its data collection state).
#####
######################################################################

class Counter:
    
    counterlist = []
    
    def __init__(self, name, value=0):
        self.value = value
        Counter.counterlist.append(self)
        
    def add(self, value=1):
        self.value += value

    def value(self):
        return value

######################################################################
#####    R E S U L T C O L L E C T O R
######################################################################
#####
#####    Statistics utility. Independent (not related to simulator and its data collection state).
#####
######################################################################

class ResultCollector:
    
    def __init__(self, scheduler, eventfile=None):
        self.scheduler = scheduler
        self.eventfile = eventfile
        self.eventfp = None
        if self.eventfile:
            self.eventfp = open(self.eventfile, "w")
        if not self.eventfp:
            self.events = []

    def close(self):
        if self.eventfp: self.eventfp.close()

    def event(self, type, event):
        if self.eventfp: 
            self.eventfp.write("%1.4f;%s;%s\n" % (self.scheduler.time(), type, event))
        else:
            self.events.append((self.scheduler.time(), type, event))

######################################################################
#####    R E P O R T E R
######################################################################
#####
#####    Statistics utility.
#####
######################################################################
class Reporter:
    
    def __init__(self, eventfile=None):
        self.multi = {}
        self.data ={}
        self.param = {}
        self.eventfile = eventfile
        self.eventfp = None
    
    def multivalue(self, key, value):
        value = float(value)
        if not self.multi.has_key(key):
            self.multi[key] = (1, value, value, value)
        else:
            self.multi[key] = (self.multi[key][0]+1, self.multi[key][1]+value, min(self.multi[key][2], value), max(self.multi[key][3], value))
            
    def value(self, key, value):
        if self.data.has_key(key): print "Warning: Reporter overwriting previous value of "+str(key)
        self.data[key] = value

    def parameter(self, key, value):
        self.param[key] = value

    def close(self):
        if self.eventfile:
            self.eventfp = open(self.eventfile, "w")
        keys = self.param.keys()
        keys.sort()
        for key in keys:
            if self.eventfp: self.eventfp.write(str(key)+";"+str(self.param[key])+"\n")
            print str(key)+";"+str(self.param[key])
        keys = self.data.keys()
        keys.sort()
        for key in keys:
            if self.eventfp: self.eventfp.write(str(key)+";"+str(self.data[key])+"\n")
            print str(key)+";"+str(self.data[key])
        keys = self.multi.keys()
        keys.sort()
        for key in keys:
            if self.multi[key][0] > 0:
                if self.eventfp: self.eventfp.write(str(key)+";"+str(self.multi[key][0])+";"+str(self.multi[key][1])+";"+str(self.multi[key][2])+";"+str(self.multi[key][3])+";"+str(self.multi[key][1]/self.multi[key][0])+"\n")
                print str(key)+";"+str(self.multi[key][0])+";"+str(self.multi[key][1])+";"+str(self.multi[key][2])+";"+str(self.multi[key][3])+";"+str(self.multi[key][1]/self.multi[key][0])
            else:
                if self.eventfp: self.eventfp.write(str(key)+";"+str(self.multi[key][0])+";"+str(self.multi[key][1])+";"+str(self.multi[key][2])+";"+str(self.multi[key][3])+";"+"\n")
                print str(key)+";"+str(self.multi[key][0])+";"+str(self.multi[key][1])+";"+str(self.multi[key][2])+";"+str(self.multi[key][3])+";"
        if self.eventfp: self.eventfp.close()
            
        
######################################################################
#####    S N A P S H O T
######################################################################
#####
#####    Statistics utility. Independent (not related to simulator and its data collection state).
#####
######################################################################

class Snapshot:
    
    def __init__(self, scheduler, interval, callback, filename):
        self.scheduler = scheduler
        self.filename = filename
        self.callback = callback
        self.fp = None
        if self.filename:
            self.fp = open(self.filename, "w")
        if not self.fp:
            self.events = []
        scheduler.every(interval, self.event)

    def event(self, dummy):
        event = str(self.callback())
        if self.fp: 
            self.fp.write("%1.4f;%s\n" % (self.scheduler.time(), event))
        else:
            self.events.append((self.scheduler.time(), event, None))

    # open issue: closing of file (currently relying on python)

######################################################################
#####    E V E N T Q U E U E
######################################################################

class EventQueue:
    
    def __init__(self):
        self.queue = []
    
    def enqueue(self, eventtime, function, parameters, repeat):
        bisect.insort(self.queue, (eventtime, function, parameters, repeat))

    def dequeue(self):
        return self.queue.pop(0)
        
    def length(self):
        return len(self.queue)

######################################################################
#####    S C H E D U L E R
######################################################################

class Scheduler:
    
    def __init__(self, until=None):
        self.until = until
        self.reset()
        
    def reset(self):
        self.eventqueue = EventQueue()
        self.simtime = 0.0
        
    def run(self):
        while 1:
            if step(): break
        
    def step(self):
        if (self.until != None and self.simtime > self.until): return True
        try:
            (eventtime, function, parameters, repeat) = self.eventqueue.dequeue()
        except IndexError:
            return True
        if eventtime < self.simtime:
            print "ERROR: Skipping event scheduled in the past (simtime="+str(self.simtime)+", eventtime="+str(eventtime)+") ",function,parameters
        else:
            self.simtime = eventtime
            if repeat:
                self.eventqueue.enqueue(self.simtime + repeat, function, parameters, repeat)
            #print function, parameters
            function(parameters)
        return False
        
    def at(self, eventtime, function, parameters=None):
        self.eventqueue.enqueue(eventtime, function, parameters, None)
        
    def after(self, eventtime, function, parameters=None):
        if eventtime < 0: 
            raise Application
            print "WARNING: Event scheduled in the past - skipping"
            return
        if eventtime == 0: 
            print "WARNING: Event scheduled immediately - accepted but unrealistic"
        self.eventqueue.enqueue(self.simtime + eventtime, function, parameters, None)
        
    def every(self, eventtime, function, parameters=None):
        self.eventqueue.enqueue(self.simtime + eventtime, function, parameters, eventtime)
        
    def time(self):
        return self.simtime
        
    def queuelength(self):
        return self.eventqueue.length()

######################################################################
#####    A P P L I C A T I O N
######################################################################

class Application:
    
    # NOT to be subclassed.
    # Subclass init() instead. 
    def __init__(self, world, node):
        self.node = node
        self.node.setapplication(self)
        self.world = world
        self.simulator = self.world.simulator
        self.init()
        
    # To be subclassed.
    def init(self):
        pass
        
    # To be subclassed.
    def update(self):
        pass

    # NOT to be subclassed.
    def send(self, msg, dest=None):
        self.node.send(msg, dest)

    # To be subclassed.
    def receive(self, msg):
        pass

    # To be subclassed.
    def getdisplayinfo(self):
        return ""

    # To be subclassed.
    def getdetailedinfo(self):
        return ""

    # To be subclassed.
    def draw(self, canvas, x, y):
        pass

######################################################################
#####    N O D E
######################################################################

class Node:
    
    nextaddress = 1
    
    def __init__(self, world, address=None, noderadius = 5, color = "#777", outline = "#000", x=None, y=None):
        if x:
            self.x = x
        else:
            self.x = random.randint(0, world.xsize)
        if y:
            self.y = y
        else:
            self.y = random.randint(0, world.ysize)
        if address == None:
            self.address = Node.nextaddress
            Node.nextaddress += 1
        else:
            self.address = address
        #self.simulator = simulator
        self.noderadius = noderadius
        self.fill = color
        self.outline = outline
        self.sendqueue = []
        self.receivequeue = []
        self.application = None
        self.links = []
        self.world = world
        self.world.nodes.append(self)
        self.counters = {}

    def __str__(self):
        return str(self.address)

    def __repr__(self):
        return "Node"+str(self.address)

    # node-specific counter
    def count(self, name, value=1):
        if not self.world.simulator.collectdata: return
        try:
            self.counters[name] += value
        except KeyError:
            self.counters[name] = value

    def count_override(self, name, value=1):
        try:
            self.counters[name] += value
        except KeyError:
            self.counters[name] = value

    def counter(self, name):
        try:
            return self.counters[name]
        except KeyError:
            return None
        
    def counterlist(self):
        return self.counters.keys()
        
    # to be subclassed
    def getdisplayinfo(self):
        return ""

    def setapplication(self, application):
        self.application = application

    def getlinks(self):
        return self.links[:]

    def getnoderadius(self):
        return self.noderadius

    def getpeers(self):
        result = []
        for link in self.links:
            result.append(link.getotherend(self))
        return result
        
    def degree(self):
        return len(self.links)
        
    def getpeerspeers(self):
        result = []
        for link in self.links:
            peer = link.getotherend(self)
            for peerspeer in peer.getpeers():
                result.append(peerspeer)
        return result
    
    def remove(self, deletelinks = True):
        assert self.world != None # already deleted
        for link in self.links[:]:      # use a copy since the structure is being modified
            if deletelinks:
                link.remove()
            else:
                link.disconnect()
        if deletelinks: assert len(self.links) == 0
        del self.world.nodes[self.world.nodes.index(self)]
        assert self not in self.world.nodes
        self.world = None
        
    def inworld(self):
        return self.world != None
        
    # graphics
    
    def draw(self, canvas):
        assert self.world
        canvas.create_oval(self.x-self.noderadius, self.y-self.noderadius, self.x+self.noderadius, self.y+self.noderadius, fill=self.fill, outline=self.outline)
        s = ""
        #for res in self.resources:
        #    s += str(res.name) + " "
        s += self.getdisplayinfo()
        if self.application: s += self.application.getdisplayinfo()
        canvas.create_text(self.x+2*self.noderadius, self.y, text=s, anchor=W)
        if self.application: self.application.draw(canvas, self.x, self.y)

    def getx(self):
        return self.x

    def gety(self):
        return self.y

    # messages
    
    def receive(self, msg):
        if not self.world: 
            #print "Lost message send to nonexisting node: to",self.address
            return
        self.count("received-msg")
        if self.application: self.application.receive(msg)
            
    def send(self, msg, dest=None, delay=DEFAULTTRANSMISSIONDELAY):
        if not self.world: 
            #print "Cannot send from nonexisting node: from",self.address,"to",dest
            return
        self.count("sent-msg")
        if dest:
            if dest == self:
                self.world.scheduler.after(0.001, dest.receive, msg)
            else:
                self.world.scheduler.after(delay, dest.receive, msg)
        else:
            for link in self.links:
                dest = link.getotherend(self)
                self.world.scheduler.after(delay, dest.receive, msg)

######################################################################
#####    L I N K 
######################################################################

class Link:
    
    def __init__(self, node1, node2, color = "#000"):
        self.node1 = node1
        self.node2 = node2
        self.world = node1.world
        assert node1 != None
        assert node2 != None
        assert node1.world == node2.world
        node1.links.append(self)
        node2.links.append(self)
        self.linkfill = color
        self.width = 1
        self.world.links.append(self)
        
    def getends(self):
        return self.node1, self.node2
        
    def getend1(self):
        return self.node1
        
    def getend2(self):
        return self.node2
    
    def getotherend(self, node):
        if self.node1 == node: return self.node2
        if self.node2 == node: return self.node1
        return None
    
    def disconnect(self):
        assert self.node1 != None
        assert self.node2 != None
        self.node1 = None
        self.node2 = None
    
    def remove(self):
        if self.world == None: return # temp
        assert self.world != None # already deleted
        del self.node1.links[self.node1.links.index(self)]
        del self.node2.links[self.node2.links.index(self)]
        self.node1 = None
        self.node2 = None
        del self.world.links[self.world.links.index(self)]
        assert self not in self.world.links
        self.world = None
        
    def inworld(self):
        return self.world != None
        
    # graphics
    
    def draw(self, canvas):
        assert self.node1 and self.node2
        canvas.create_line(self.node1.getx(), self.node1.gety(), self.node2.getx(), self.node2.gety(), fill=self.linkfill, width=self.width)
        self.width = 1
        
    def flash(self):
        self.width = 3

    # resources
    
######################################################################
#####    W O R L D 
######################################################################

class World:
    
    def __init__(self, xsize, ysize):
        self.simulator = None
        self.scheduler = None
        self.xsize = xsize
        self.ysize = ysize
        self.clear()
        
    def clear(self):
        self.nodes = []
        self.links = []
        
    def randomnode(self):
        return self.nodes[random.randint(0, len(self.nodes)-1)]
    
    def randomlink(self):
        return self.links[random.randint(0, len(self.links)-1)]

    def randomcoordinate(self):
        return (random.randint(0, self.xsize), random.randint(0, self.ysize))

    def randomcoordinateclose(self, x, y, dist):
        randx = random.randint(max(x-dist*self.xsize, 0), min(x+dist*self.xsize, self.xsize))
        randy = random.randint(max(y-dist*self.ysize, 0), min(y+dist*self.ysize, self.ysize))
        return (randx, randy)
        
    def getnodes(self):
        return self.nodes[:]

    def getlinks(self):
        return self.links[:]

    def draw(self, canvas):
        canvas.delete(ALL)
        for link in self.links:
            link.draw(canvas)
        for node in self.nodes:
            node.draw(canvas)

    def nodecount(self):
        return len(self.nodes)

    def linkcount(self):
        return len(self.links)

    def closest(self, x, y):
        closest = None
        closestdist = 2*(self.xsize+self.ysize)
        for node in self.nodes:
            d = math.sqrt((x - node.x)*(x - node.x) + (y - node.y)*(y - node.y))
            if d < closestdist: 
                closest = node
                closestdist = d
        return closest

    def isconnected(self):
        list = [self.nodes[0]]
        found = []
        while len(list) > 0:
            node = list.pop(0)
            found.append(node)
            for neighbor in node.getpeers():
                if (neighbor not in list) and (neighbor not in found):
                    list.append(neighbor)
        if len(found) == len(self.nodes):
            print "Connected"
            return True
        print "Not connected"
        return False

    def makeconnectedworld(self, nodes, links, nodecolor="#000", linkcolor="#000"):
        while 1:
            self.clear()
            self.makelinkcontrolledworld(nodes, links, nodecolor, linkcolor)
            if self.isconnected(): break

    def makelinkcontrolledworld(self, nodes, links, nodecolor="#000", linkcolor="#000"):
        assert links > nodes
        for i in range(nodes):
            newnode = Node(self, color = nodecolor)
        if nodes == 1 or links == 0: return
        for i in range(links):
            while 1:
                peer1 = self.randomnode()
                peer2 = self.randomnode()
                if peer1 != peer2: break
            newlink = Link(peer1, peer2, color=linkcolor)
        while 1:
            foundunconnected = False
            for peer in self.getnodes():
                if peer.degree() == 0:
                    foundunconnected = True
                    link = self.randomlink()
                    self.removelink(link)
                    while 1:
                        peer2 = self.randomnode()
                        if peer != peer2: break
                    self.addlink(peer, peer2, color=linkcolor)
            if not foundunconnected: break

    
######################################################################
#####    S I M U L A T I O N
######################################################################

class Simulation:
    
    ##### INITIALIZATION
    
    STATE_IDLE = 1
    STATE_RUNNING = 2
    STATE_PAUSE = 3
    
    def __init__(self, scheduler, master, scenariomanager):
        self.sim_mutex = thread.allocate_lock()
        self.world = None       # assigned by setworld
        self.master = master
        self.scenariomanager = scenariomanager
        self.scheduler = scheduler
        self.infowindow = None
        self.exceptionwindow = None
        self.selectednode = None
        self.state = self.STATE_IDLE
        self.eventspersecond = 0
        self.lastinfotext = ""
        self.stoprequest = False
        self.pauserequest = False
        self.counters = {}
        self.outputdir = ".\\"
        
        # parameters selected through UI:
        self.redrawinterval = 0.2
        self.delay = 0
        
    def setworld(self, world):
        self.world = world
        world.simulator = self
        world.scheduler = self.scheduler
        self.initscreen()

    def setoutputdir(self, outputdir):
        self.outputdir = outputdir

    ##### DRAWING
    
    def initscreen(self):
        self.frame = Frame(self.master)
        self.frame.pack()
        self.master.title("PONGsim")
        
        rightframe = Frame(self.frame)
        rightframe.pack(side=RIGHT)
        
        buttonframe = Frame(rightframe)
        buttonframe.pack(side=TOP)
        self.label1 = Label(buttonframe, text="Running scenario:", relief=RIDGE, width=25)
        self.label1.pack(side=TOP)
        self.scenariolabel= Label(buttonframe, text="(None)")
        self.scenariolabel.pack(side=TOP)
        self.label1 = Label(buttonframe, text="Output file:", relief=RIDGE, width=25)
        self.label1.pack(side=TOP)
        self.filelabel= Label(buttonframe, text="(None)")
        self.filelabel.pack(side=TOP)
        self.label1 = Label(buttonframe, text="Simulated time:", relief=RIDGE, width=25)
        self.label1.pack(side=TOP)
        self.timelabel = Label(buttonframe, text="0.00s (0h 0m 0s)")
        self.timelabel.pack(side=TOP)
        self.label1 = Label(buttonframe, text="Events per second:", relief=RIDGE, width=25)
        self.label1.pack(side=TOP)
        self.eventsperseclabel= Label(buttonframe, text="0")
        self.eventsperseclabel.pack(side=TOP)
        self.label1 = Label(buttonframe, text="Event queue length:", relief=RIDGE, width=25)
        self.label1.pack(side=TOP)
        self.queuelenlabel= Label(buttonframe, text="0")
        self.queuelenlabel.pack(side=TOP)
        #self.label1 = Label(buttonframe, text="Nodes:", relief=RIDGE, width=25)
        #self.label1.pack(side=TOP)
        #self.nodecountlabel = Label(buttonframe, text="0")
        #self.nodecountlabel.pack(side=TOP)
        #self.label1 = Label(buttonframe, text="Links:", relief=RIDGE, width=25)
        #self.label1.pack(side=TOP)
        #self.linkcountlabel = Label(buttonframe, text="0")
        #self.linkcountlabel.pack(side=TOP)
        self.label3 = Label(buttonframe, text=" ")
        self.label3.pack(side=TOP)
        self.canvas = Canvas(self.frame, height=self.world.ysize, width=self.world.xsize, bg="white")
        
        scenarioframe = Frame(buttonframe)
        scenarioframe.pack(side=TOP)
        self.label1 = Label(scenarioframe, text="Available scenarios:", width=25)
        self.label1.pack(side=TOP)
        self.yScroll  =  Scrollbar (scenarioframe, orient=VERTICAL)
        self.yScroll.pack(side=RIGHT, fill=Y)
        self.listbox = Listbox (scenarioframe, yscrollcommand=self.yScroll.set, selectmode=MULTIPLE)
        self.listbox.pack(side=RIGHT, fill=X)
        scenarios = self.scenariomanager.getscenarios()
        scenarios.sort()
        for scenario in scenarios:
            self.listbox.insert(END, scenario)
        self.yScroll["command"]  =  self.listbox.yview
        self.listbox.bind("<ButtonRelease-1>", self.button_selectscenario)
        self.label5 = Label(buttonframe, text="Run times:", width=25)
        self.label5.pack(side=TOP)
        self.timesentry = Entry(buttonframe, width=5)
        self.timesentry.insert("1", 1)
        self.timesentry.pack(side=TOP)
        self.redrawintervalslider = Scale(buttonframe, label="Redraw interval (s)", resolution=0.10, orient=HORIZONTAL, sliderlength=20, length=140, from_=0.10, to=2, command=self.setredrawinterval)
        self.redrawintervalslider.pack(side=TOP)
        self.redrawintervalslider.set(self.redrawinterval)
        self.delayslider = Scale(buttonframe, label="Step delay (s)", resolution=0.0005, orient=HORIZONTAL, sliderlength=20, length=140, from_=0.0, to=0.00500, command=self.setdelay)
        self.delayslider.pack(side=TOP)
        self.delayslider.set(self.delay)
        self.label4 = Label(buttonframe, text=" ")
        self.label4.pack(side=TOP)
        #self.button4 = Button(buttonframe, text="Reset", width=25, command=self.button_reset)
        #self.button4.pack(side=TOP)
        self.button1 = Button(buttonframe, text="Run", width=25, command=self.button_start)
        self.button1.pack(side=TOP)
        self.button5 = Button(buttonframe, text="Stop", width=25, command=self.button_stop)
        self.button5.pack(side=TOP)
        self.button2 = Button(buttonframe, text="Step", width=25, command=self.button_step)
        self.button2.pack(side=TOP)
        self.button3 = Button(buttonframe, text="Quit", fg="red", width=25, command=self.exit)
        self.button3.pack(side=TOP)
        self.canvas.bind("<Button-1>", self.clickleft)
        self.canvas.bind("<Button-2>", self.clickright)
        self.canvas.pack(side=TOP)

        self.master.after(1, self.redraw)
    
    # loop forever
    def redraw(self):
        self.sim_mutex.acquire()
        self.draw()
        self.sim_mutex.release()
        self.master.after(int(self.redrawinterval*1000), self.redraw) # give time to Tk to update window

    def draw(self, dummy=None):
        self.world.draw(self.canvas)
        if self.selectednode and self.selectednode.inworld(): self.canvas.create_oval(self.selectednode.getx()-self.selectednode.getnoderadius()-2, self.selectednode.gety()-self.selectednode.getnoderadius()-2, self.selectednode.getx()+self.selectednode.getnoderadius()+2, self.selectednode.gety()+self.selectednode.getnoderadius()+2, outline="#F00")
        self.updatestatus()

    def updatestatus(self):
        self.timelabel.config(text="%1.2fs (%1dh %1dm %1ds)" % (self.scheduler.time(), int(self.scheduler.time() / 3600), int((self.scheduler.time() % 3600) /60), self.scheduler.time() % 60))
        #self.nodecountlabel.config(text=str(self.world.nodecount()))
        #self.linkcountlabel.config(text=str(self.world.linkcount()))
        self.eventsperseclabel.config(text=str(int(self.eventspersecond)))
        self.queuelenlabel.config(text=str(int(self.scheduler.queuelength())))
        if self.state == self.STATE_RUNNING or self.state == self.STATE_PAUSE:
            scenario = self.scenariomanager.getactivescenario().getname() + "  ("+str(self.scenariomanager.getrunnumber())+")"
            filename = self.scenariomanager.getactivescenario().getfilename()
        else:
            scenario = "(None)"
            filename = "(None)"
        self.scenariolabel.config(text=scenario)
        self.filelabel.config(text=filename)

        if self.selectednode:
            if self.selectednode.application:
                if self.infowindow:
                    t = "Node: "+str(self.selectednode.address)+"\n"
                    t = t + self.selectednode.application.getdetailedinfo()
                    if t != self.lastinfotext:
                        try:
                            # self.infolabel.config(text=t)
                            self.infotext.delete("1.0", END)
                            self.infotext.insert(END, t)
                        except TclError:
                            pass
                    self.lastinfotext = t

    ##### SIMULATOR

    # Runs the simulator in a separate thread.
    # Externally changed variables: self.stoprequest, self.pausrequest, self.delay
    def runsim(self, dummy):
        try:
            self.state = self.STATE_RUNNING
            try:
                runtimes = int(self.timesentry.get())
            except ValueError:
                runtimes = 1
            self.scenariomanager.resetqueue()
            for i in range(runtimes):
                items = self.listbox.curselection()
                try:
                    items = map(string.atoi, items)
                except ValueError: 
                    pass
                for selection in items:
                    self.scenariomanager.enqueue(self.listbox.get(selection), i+1)
            print "Start simulation:",self.scenariomanager.count(),"scenarios, each",runtimes,"times"
            activescenario = self.scenariomanager.pop()
            lastpasstime = None
            while activescenario != None and not self.stoprequest: 
                print "Start scenario:",activescenario.getname()
                try:
                    self.reset()
                except:
                    self.exception("initialization")
                stop = False
                events = 0
                while not stop:
                    while self.pausrequest:
                        time.sleep(1)
                        lastpasstime = None
                    self.sim_mutex.acquire()
                    for ev in range(100):
                        if self.delay > 0: time.sleep(self.delay)
                        try:
                            outofevents = self.scheduler.step()
                        except:
                            self.exception("event handling in user code")
                        if outofevents: 
                            stop = True
                            print "Finish scenario: no events left"
                            break
                        if self.stopcriteria():
                            stop = True
                            print "Finish scenario: stop criteria"
                            break
                        if self.stoprequest or self.pausrequest:
                            if self.stoprequest:
                                stop = True
                                print "Abort scenario: stop requested"
                            break
                        events += 1
                    if lastpasstime and (time.time()-lastpasstime >0):
                        eventspersec = events / (time.time()-lastpasstime)
                        self.eventspersecond = self.eventspersecond*0.70 + 0.30*(eventspersec)
                    lastpasstime = time.time()
                    self.sim_mutex.release()
                self.close()
                self.reporter.value("simulation-time", self.scheduler.time())
                self.reporter.close()
                activescenario = self.scenariomanager.pop()
            if self.stoprequest:
                print "Abort simulation: remaining scenarios skipped"
            else:
                print "Finish simulation: all scenarios finished"
            self.state = self.STATE_IDLE
            self.button1.config(text="Run")
            self.stoprequest = False
        except:
            self.exception("simulator framework during simulator run")

    def exception(self, where):
        text = string.joinfields(traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback), "\n")
        text = "ERROR in " + where + "\n" + "-"*80+ "\n" + text + "-"*80+ "\nSimulation paused - press Continue to ignore and continue"
        print text
        self.closeexceptionwin()
        self.exceptionwindow = Toplevel(height=400, width=500)
        frame = Frame(self.exceptionwindow, relief=RIDGE)
        frame.pack(side=LEFT)
        label = Label(frame, text=text, height=40, width=80, justify=LEFT)
        label.pack(side=LEFT)    
        self.state = self.STATE_PAUSE
        self.button1.config(text="Continue")
        self.pausrequest = True
            
    def closeexceptionwin(self):
        if self.exceptionwindow:
            try:
                self.exceptionwindow.withdraw()
            except TclError:
                pass
            self.exceptionwindow = None
            
    # Prepare simulating a scenario. 
    def reset(self):
        print "Initialization: start"
        self.scheduler.reset()
        self.eq = EventQueue()
        reportname = self.outputdir + str(self.scenariomanager.getactivescenario().getfilename())+" "+time.strftime("%Y%m%d %H%M")+".output.txt"
        self.reporter = Reporter(reportname)
        params = self.scenariomanager.getactivescenario().getall()
        for key in params.keys():
            self.reporter.parameter(key, params[key])
        self.world.clear()
        self.collectdata = True
        self.eventspersecond = 0
        self.selectednode = None
        self.init()
        print "Initialization: finish"

    ##### SUBCLASSED FUNCTIONS

    # To be overwritten by subclass
    # Returns True if the simulation should stop (e.g. based on time or even count)
    def stopcriteria(self):
        return False

    def init(self):
        # to be subclassed
        pass

    def close(self):
        # to be subclassed
        pass

    ##### USER INTERFACE EVENTS

    def exit(self):
        self.closeinfowin()
        self.closeexceptionwin()
        self.frame.quit()

    def button_start(self):
        self.closeexceptionwin()
        self.closeinfowin()
        if self.state == self.STATE_IDLE and not self.stoprequest:          # Start
            self.state = self.STATE_RUNNING
            self.button1.config(text="Pause")
            self.pausrequest = False
            self.stoprequest = False
            thread.start_new_thread(self.runsim, (self,))
        elif self.state == self.STATE_PAUSE and not self.stoprequest:    # Resume
            self.state = self.STATE_RUNNING
            self.button1.config(text="Pause")
            self.pausrequest = False
        elif self.state == self.STATE_RUNNING and not self.stoprequest:  # Pause
            self.state = self.STATE_PAUSE
            self.button1.config(text="Continue")
            self.pausrequest = True

    def button_stop(self):
        self.closeexceptionwin()
        self.closeinfowin()
        if self.state == self.STATE_RUNNING or self.state == self.STATE_PAUSE:
            print "Stopping..."
            self.stoprequest = True

    def button_step(self):
        self.closeexceptionwin()
        self.closeinfowin()
        if self.state == self.STATE_PAUSE:
            print "Step"
            self.scheduler.step()
               
    def setredrawinterval(self, event):
        self.redrawinterval = self.redrawintervalslider.get()

    def setdelay(self, event):
        self.delay = self.delayslider.get()

    def button_selectscenario(self, event):
        pass
        
    def clickright(self, event):
        node = self.world.closest(event.x, event.y)
        if node: 
            self.selected = node
            self.draw() # update info since automatic redrawing is disable
    
    def clickleft(self, event):
        if self.infowindow == None: 
            self.infowindow = Toplevel(height=60, width=40)
            self.infoframe = Frame(self.infowindow, relief=RIDGE)
            self.infoframe.pack(side=LEFT)
            #self.infolabel = Label(self.infoframe, text="Waiting for info...\n\n", height=60, width=60, justify=LEFT)
            #self.infolabel.pack(side=LEFT)
            self.infotext = Text(self.infoframe, height=60, width=60, wrap=WORD)
            self.infotext.pack(side=LEFT)
            self.infotext.insert(END, "Waiting for info...")
            scroll = Scrollbar(self.infoframe, orient=VERTICAL)
            scroll.pack(side=RIGHT, fill=Y)
            scroll.config(command=self.infotext.yview)
            #self.infowindow.bind("<Destroy>", self.closeinfowin)
            self.infotext["yscrollcommand"]  =  scroll.set
        self.selectednode = self.world.closest(event.x, event.y)
    
    def closeinfowin(self, event=None):    
        if self.infowindow:
            try:
                self.infowindow.withdraw()
            except TclError:
                pass
            self.infowindow = None

    ##### SIMULATOR-SPECIFIC COUNTER
    
    def count(self, name, value=1):
        if not self.collectdata: return
        try:
            self.counters[name] += value
        except KeyError:
            self.counters[name] = value

    def counter(self, name):
        try:
            return self.counters[name]
        except KeyError:
            return None
    
    def counterlist(self):
        return self.counters.keys()

if __name__ == '__main__':
    print "This is a framework for p2p simulations, and is not runnable as such."
    print "To use it, create subclasses of the framework classes."
    