######################################################################
#####
#####    PONGsim (Python Overlay Network Graphical Simulator)
#####    Topology generator module
#####    Version: 1.00
#####
######################################################################
#####    
#####    PONGsim (Python Overlay Network Graphical Simulator)
#####    
#####    Copyright (c) 2005-2007 Nicklas Beijar
#####    
#####    Permission is hereby granted, free of charge, to any person
#####    obtaining a copy of this software and associated documentation
#####    files (the "Software"), to deal in the Software without
#####    restriction, including without limitation the rights to use,
#####    copy, modify, merge, publish, distribute, sublicense, and/or
#####    sell copies of the Software, and to permit persons to whom the
#####    Software is furnished to do so, subject to the following
#####    conditions:
#####    
#####    The above copyright notice and this permission notice shall be
#####    included in all copies or substantial portions of the Software.
#####    
#####    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#####    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
#####    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#####    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#####    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
#####    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#####    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
#####    OTHER DEALINGS IN THE SOFTWARE.
#####
######################################################################
#####
#####    Nicklas Beijar, nbeijar@netlab.tkk.fi
#####    Networking Laboratory, Helsinki University of Technology
##### 
######################################################################
#####
#####    To do:
#####    - (nothing)
#####
######################################################################

import random

class Topology:
    
    def __init__(self, nodes):
	self.nodes = nodes
	self.edges = {}
    
    def hasedge(self, a, b):
	try:
	    if self.edges[a][b] != 0: return True
	except KeyError:
	    pass
	return False

    def addedge(self, a, b, weight = 1):
	if not self.edges.has_key(a):
	    self.edges[a] = {}
	self.edges[a][b] = weight

    # Problems:
    # - not always connected
    # - does not always generate an even M -> does not stop
    # - risk that the last possible link is impossible because of existing link -> does not stop
    def powerlaw_PLOD(self, alpha, beta):
	degree = {}
	M = 0
	for i in range(self.nodes):
	    x = random.randint(1, self.nodes)
	    degree[i] = round(beta*pow(x, -alpha))
	    print degree[i]
	    M += degree[i]
	for i in range(int(M/2)):
	    while 1:
		r = random.randint(1, self.nodes) -1
		c= random.randint(1, self.nodes) -1
		if r != c and not self.hasedge(r, c) and not self.hasedge(c, r):
		    if degree[r] > 0 and degree[c] > 0:
			degree[r] -= 1
			degree[c] -= 1
			self.addedge(r, c)
			self.addedge(c, r)
			print i, M
			break

    def powerlaw_BA_based(self, minlinks):
	degree = {}
	for i in range(self.nodes):
	    degree[i] = 1
	for i in range(self.nodes):
	    for j in range(minlinks):
                totsum = 0
                for k in range(self.nodes):
                    try:
                        totsum += degree[k]
                    except KeyError:
                        totsum += 0
                r = random.randint(0, totsum-1)
                totsum = 0
                for k in range(self.nodes):
                    totsum += degree[k]
                    if r < totsum: break
                if i != k and not self.hasedge(i, k) and not self.hasedge(k, i):
                    self.addedge(i, k)
                    self.addedge(k, i)
                    degree[i] += 1
                    degree[k] += 1

    def powerlaw_BA_degree(self, mindegree):
	degree = {}
	for i in range(self.nodes):
	    degree[i] = 1
        noadded = 0
        while 1:
            degreesum = 0.0
            for i in range(self.nodes):
                degreesum += degree[i]
                totsum = 0
                for k in range(self.nodes):
                    try:
                        totsum += degree[k]
                    except KeyError:
                        totsum += 0
                r = random.randint(0, totsum-1)
                totsum = 0
                for k in range(self.nodes):
                    totsum += degree[k]
                    if r < totsum: break
                if i != k and not self.hasedge(i, k) and not self.hasedge(k, i):
                    self.addedge(i, k)
                    self.addedge(k, i)
                    degree[i] += 1
                    degree[k] += 1
                    noadded = 0
                else:
                    noadded += 1
            if degreesum/self.nodes >= mindegree or noadded > self.nodes*20: break
    
    def fullyconnected(self):
        for i in range(self.nodes):
            for j in range(self.nodes):
                if i != j:
                    self.addedge(i, j)

    def list(self):
	print "Listing:"
	for a in self.edges.keys():
	    for b in self.edges[a]:
		print a, b, self.edges[a][b]
                
    def avgdegree(self):
        sum = 0
        count = 0.0
	for a in self.edges.keys():
	    sum += len(self.edges[a])
            count += 1.0
        return sum/count

    def neighbors(self, a):
        n = []
        for b in self.edges[a].keys():
            n.append(b)
        return n

    def isconnected(self):
        list = [1,]
        found = []
        while len(list) > 0:
            node = list.pop(0)
            found.append(node)
            for neighbor in self.edges[node]:
                if (neighbor not in list) and (neighbor not in found):
                    list.append(neighbor)
        if len(found) == self.nodes:
            return True
        return False

if __name__ == '__main__':
    print "powerlaw_BA_based:"
    while 1:
        print "Generating topology"
        t = Topology(17)
        t.powerlaw_BA_based(2)
        if t.isconnected(): break
    print "Connected power-law topology"
    t.list()
    print "Average degree:",t.avgdegree()

    print "powerlaw_BA_degree:"
    while 1:
        print "Generating topology"
        t = Topology(17)
        t.powerlaw_BA_degree(3)
        if t.isconnected(): break
    print "Connected power-law topology"
    t.list()
    print "Average degree:",t.avgdegree()

    print "fully connected:"
    while 1:
        print "Generating topology"
        t = Topology(17)
        t.fullyconnected()
        if t.isconnected(): break
    print "Fully connected topology"
    t.list()
    print "Average degree:",t.avgdegree()
    
